/******************************************************************************
 * spanning_tree.h
 *
 * Implementation of the spanning tree function. This function is based in a DFS
 * traversal of the input graph
 * 
 * Compile : gcc -o st main.c defs.c spanning_tree.c util.c
 * Usage   : st <input graph> <output tree>
 *
 ******************************************************************************
 * Copyright (C) 2016 José Fuentes Sepúlveda <jfuentess@dcc.uchile.cl>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "spanning_tree.h"

Tree *dfs_spanning_tree(Graph *g) {
  Tree *t = create_tree(g->n);
  
  char *visited = (char *)calloc(g->n, sizeof(char));
  uint *edges = (uint *)calloc(2*g->m,sizeof(uint));
  int *parent = malloc(g->n*sizeof(int));
  
  int i = 0, j = 0;
  
  Stack *s = stack_create(g->n);

  // Starting vertex 0
  visited[0] = 1;
  stack_push(s, 0);
  parent[0] = -1;

  // Set the parent of each node
  while(!stack_empty(s)) {
    int curr = (int)stack_pop(s);

    for(i = g->V[curr].first; i <= g->V[curr].last; i++) {
      int tgt = g->E[i].tgt;
      
      if(!visited[tgt]) { // Not visited
	visited[tgt] = 1;
	stack_push(s, tgt);
	parent[tgt] = g->E[i].cmp; // Edge child-to-parent
      }
    }
  }

  // Select the edges that will be part of the spanning tree
  for(i = 0; i < g->n; i++) {
    edges[parent[i]] = 1;
    edges[g->E[parent[i]].cmp] = 1;
  }

  // Define the fields src and tgt for the edges and first and last for the
  // nodes
  unsigned int m = 0;
  for(i = 0; i < g->n; i++) {
    unsigned int first = g->V[i].first;
    unsigned int last = g->V[i].last;
    int init = parent[i];

    // Starting vertex
    if(i == 0)
      init = 0;
    
    for(j = first; j <= last; j++, init++) {
      if(init > last)
	init = first;
      
      if(edges[init]) {
	t->E[m].src = g->E[init].src;
	t->E[m].tgt = g->E[init].tgt;

	if(m == 0)
	  t->N[t->E[m].src].first = m;
	else if(t->E[m].src != t->E[m-1].src) {
	  t->N[t->E[m-1].src].last = m-1;
	  t->N[t->E[m].src].first = m;	  
	}
	m++;
      }
    }

    t->N[t->E[m-1].src].last = m-1;
  }

  // Define the field cmp for the edges
  for(i = 0; i < 2*(t->n-1); i++) {
    Node tgt = t->N[t->E[i].tgt];
    
    for(j = tgt.first; j <= tgt.last; j++)
      if(t->E[j].tgt == t->E[i].src) {
  	t->E[i].cmp = j;
  	break;
      }
  }

  // Validate the tree
  /* if(tree_cycle(t)) */
  /*   printf("t IS NOT a tree!\n"); */
  /* else */
  /*   printf("t IS a tree\n"); */
  
  return t;
}
